/**=================================================================================================
Hochschule Luzern - Technik & Architektur                                   Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer                                                     http://hslu.ximit.ch
----------------------------------------------------------------------------------------------------
Project     : Adressverwaltung
Name        : address.h
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
Version     : 2012-10-19v1.0
Description : Adressverwaltung
==================================================================================================*/
#ifndef ADDRESS_H
#define	ADDRESS_H

#define PATH_SIZE 255 // File Path Buffer Size
#define TRUE	  1   // Boolean True
#define	FALSE	  0   // Boolean False

/**
 * Custom Error Return Code
 */
typedef enum errorCode {
    SUCCESS           = 1,   // OK
    UNKNOW            = 0,   // undef status
    ERROR             = -1,  // general error
    NOMEMORY          = -2,  // alloc, realloc failed
    FILECREATEERROR   = -11, // fopen failed
    FILECLOSEERROR    = -12, // fclose failed
    FILENOTFOUNDERROR = -13, // file not found
    FILEREADERROR     = -14  // file read failed
} err_t;

/**
 * Defines address struct
 */
#define MAXSTRLEN 45 // Max String Len 
typedef struct address_ {
    char firstname[MAXSTRLEN+1];
    char lastname[MAXSTRLEN+1];
    char street[MAXSTRLEN+1];
    unsigned int zip;
    char city[MAXSTRLEN+1];
    unsigned int index;
} address_t, address_p;

/**
 * Defines address List
 */
typedef struct addressList_ {
    address_t* addr;
    int size;
} addressList_t, addressList_p;

/**
 * Init AddressList (Allocate Memory)
 * @return Success / Error Status Code
 */
err_t initialize(void);

/**
 * List the current Address List (with CallBack Function for Output)
 * @param showAddrEntryFN CallBack Function for Output
 */
void listAddress(void (*showAddrEntryFN)(int, address_t*));

/**
 * Add a new Address to the List (Values)
 * @param lastname Lastname of the new entry
 * @param firstname Firstname of the new entry
 * @param street Street of the new entry
 * @param zip Zip of the new entry
 * @param city City of the new entry
 * @return Success / Error Status Code
 */
err_t addAddressValues(char* lastname, char* firstname, char* street, int zip, char* city);

/**
 * Add a new Address to the list (Address Object Pointer)
 * @param newAddress Pointer to the new Address
 * @return Success / Error Status Code
 */
err_t addAddress(address_p newAddress);

/**
 * Clear Address List in the Memory (free & re-malloc)
 * @return Success / Error Status Code
 */
err_t clearAddress(void);

/**
 * Save the current AddressList to a File
 * @param path Path to File for save
 * @param num Number or Addresses Loaded
 * @return Success / Error Status Code
 */
err_t saveAddrFile(char* path, int* num);

/**
 * Load Addresses from a File and them to the current AddressList
 * @param path Path to File for reading
 * @param num Number or Addresses Loaded
 * @return Success / Error Status Code
 */
err_t readAddrFile(char* path, int* num);

/**
 * Defines SortMode
 */
typedef enum { NAME=1, STREET, ZIP, CITY} sortmode;  // SortMode

/**
 * Sort Address by Sortmode with a modified (stable!) QuickSort
 * @param mode Sortmode, i.e. NAME, STREET, ZIP or CITY
 * @return Success / Error Status Code
 */
err_t sortAddress(sortmode mode);

/**
 * Compare Function pointer
 */
typedef int (*compfn)(const void *a, const void *b);

/**
 * Comparator by Name
 * @param a Address 1
 * @param b Address 2
 * @return Compare Result, >0 if Addr2 is bigger than Addr1
 */
int compareByName(const void *a, const void *b);

/**
 * Comparator by Street
 * @param a Address 1
 * @param b Address 2
 * @return Compare Result, >0 if Addr2 is bigger than Addr1
 */
int compareByStreet(const void *a, const void *b);

/**
 * Comparator by Zip
 * @param a Address 1
 * @param b Address 2
 * @return Compare Result, >0 if Addr2 is bigger than Addr1
 */
int compareByZip(const void *a, const void *b);

/**
 * Comparator by City
 * @param a Address 1
 * @param b Address 2
 * @return Compare Result, >0 if Addr2 is bigger than Addr1
 */
int compareByCity(const void *a, const void *b);

#endif	/* ADDRESS_H */
